from ape import plugins
from ape.api import NetworkAPI, create_network_type
from ape.api.networks import LOCAL_NETWORK_NAME
from ape_geth import GethProvider
from ape_test import LocalProvider

from .ecosystem import NETWORKS, BSC, BSCConfig


@plugins.register(plugins.Config)
def config_class():
    return BSCConfig


@plugins.register(plugins.EcosystemPlugin)
def ecosystems():
    yield BSC


@plugins.register(plugins.NetworkPlugin)
def networks():
    for network_name, network_params in NETWORKS.items():
        yield "bsc", network_name, create_network_type(*network_params)

    # NOTE: This works for development providers, as they get chain_id from themselves
    yield "bsc", LOCAL_NETWORK_NAME, NetworkAPI
    yield "bsc", "mainnet-fork", NetworkAPI


@plugins.register(plugins.ProviderPlugin)
def providers():
    for network_name in NETWORKS:
        yield "bsc", network_name, GethProvider

    yield "bsc", LOCAL_NETWORK_NAME, LocalProvider



from ape.api.config import PluginConfig
from ape.api.networks import LOCAL_NETWORK_NAME
from ape_ethereum.ecosystem import Ethereum, NetworkConfig

NETWORKS = {
    # chain_id, network_id
    "mainnet": (56, 56),
    "testnet": (97, 97),
}


class BSCConfig(PluginConfig):
    mainnet: NetworkConfig = NetworkConfig(required_confirmations=1, block_time=3)  # type: ignore
    testnet: NetworkConfig = NetworkConfig(required_confirmations=1, block_time=3)  # type: ignore
    local: NetworkConfig = NetworkConfig(default_provider="test")  # type: ignore
    default_network: str = LOCAL_NETWORK_NAME


class BSC(Ethereum):
    @property
    def config(self) -> BSCConfig:  # type: ignore
        return self.config_manager.get_config("bsc")  # type: ignore
